@@ -14,7 +14,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2013 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2014 Live Networks, Inc.  All rights reserved.
 // A sink that generates an AVI file from a composite media session
 // Implementation
 
@@ -102,7 +102,6 @@ public:
   AVIIndexRecord(unsigned chunkId, unsigned flags, unsigned offset, unsigned size)
     : fNext(NULL), fChunkId(chunkId), fFlags(flags), fOffset(offset), fSize(size) {
   }
-  virtual ~AVIIndexRecord() { delete fNext; }
 
   AVIIndexRecord*& next() { return fNext; }
   unsigned chunkId() const { return fChunkId; }
@@ -175,10 +174,12 @@ AVIFileSink::AVIFileSink(UsageEnvironment& env,
 AVIFileSink::~AVIFileSink() {
   completeOutputFile();
 
-  // Then, delete each active "AVISubsessionIOState":
+  // Then, stop streaming and delete each active "AVISubsessionIOState":
   MediaSubsessionIterator iter(fInputSession);
   MediaSubsession* subsession;
   while ((subsession = iter.next()) != NULL) {
+    subsession->readSource()->stopGettingFrames();
+
     AVISubsessionIOState* ioState
       = (AVISubsessionIOState*)(subsession->miscPtr);
     if (ioState == NULL) continue;
@@ -187,7 +188,12 @@ AVIFileSink::~AVIFileSink() {
   }
 
   // Then, delete the index records:
-  delete fIndexRecordsHead;
+  AVIIndexRecord* cur = fIndexRecordsHead;
+  while (cur != NULL) {
+    AVIIndexRecord* next = cur->next();
+    delete cur;
+    cur = next;
+  }
 
   // Finally, close our output file:
   CloseOutputFile(fOutFid);
