@@ -14,12 +14,12 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2013 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2014 Live Networks, Inc.  All rights reserved.
 // A class used for digest authentication.
 // Implementation
 
 #include "DigestAuthentication.hh"
-#include "our_md5.h"
+#include "ourMD5.hh"
 #include <strDup.hh>
 #include <GroupsockHelper.hh> // for gettimeofday()
 #include <stdio.h>
@@ -125,7 +125,7 @@ char const* Authenticator::computeDigestResponse(char const* cmd,
 }
 
 void Authenticator::reclaimDigestResponse(char const* responseStr) const {
-  free((char*)responseStr); // NOT delete, because it was malloc-allocated
+  delete[](char*)responseStr;
 }
 
 void Authenticator::resetRealmAndNonce() {
