@@ -14,7 +14,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2013 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2014 Live Networks, Inc.  All rights reserved.
 // A 'ServerMediaSubsession' object that creates new, unicast, "RTPSink"s
 // on demand, from an H264 video track within a Matroska file.
 // Implementation
@@ -28,65 +28,63 @@ H264VideoMatroskaFileServerMediaSubsession* H264VideoMatroskaFileServerMediaSubs
   return new H264VideoMatroskaFileServerMediaSubsession(demux, trackNumber);
 }
 
-#define checkPtr if (ptr >= limit) return;
-#define numBytesRemaining (unsigned)(limit - ptr)
+#define CHECK_PTR if (ptr >= limit) return
+#define NUM_BYTES_REMAINING (unsigned)(limit - ptr)
 
 H264VideoMatroskaFileServerMediaSubsession
 ::H264VideoMatroskaFileServerMediaSubsession(MatroskaFileServerDemux& demux, unsigned trackNumber)
   : H264VideoFileServerMediaSubsession(demux.envir(), demux.fileName(), False),
     fOurDemux(demux), fTrackNumber(trackNumber),
     fSPSSize(0), fSPS(NULL), fPPSSize(0), fPPS(NULL) {
-  // Use our track's 'Codec Private' data: Byte 4 contains the size of NAL unit lengths,
-  // and bytes 5 and beyond contain SPS and PPSs:
-  MatroskaTrack* track = fOurDemux.lookup(fTrackNumber);
-  if (track->codecPrivateSize >= 5) track->subframeSizeSize = (track->codecPrivate[4])&0x3 + 1;
-
+  // Use our track's 'Codec Private' data: Bytes 5 and beyond contain SPS and PPSs:
   unsigned numSPSandPPSBytes;
   u_int8_t* SPSandPPSBytes;
+  MatroskaTrack* track = fOurDemux.lookup(fTrackNumber);
+
   if (track->codecPrivateSize >= 6) {
     numSPSandPPSBytes = track->codecPrivateSize - 5;
-    track->codecPrivate[5] &=~ 0xE0; // mask out the top 3 (reserved) bits from the 'numSPSs' byte
     SPSandPPSBytes = &track->codecPrivate[5];
   } else {
     numSPSandPPSBytes = 0;
     SPSandPPSBytes = NULL;
   }
 
-  // Extract, from "SPSandPPSBytes", one SPS NAL unit, and one PPS NAL unit.  (I hope one is all we need of each.)
-  do {
-    if (numSPSandPPSBytes == 0 || SPSandPPSBytes == NULL) break; // sanity check
-    u_int8_t* ptr = SPSandPPSBytes;
-    u_int8_t* limit = &SPSandPPSBytes[numSPSandPPSBytes];
+  // Extract, from "SPSandPPSBytes", one SPS NAL unit, and one PPS NAL unit.
+  // (I hope one is all we need of each.)
+  if (numSPSandPPSBytes == 0 || SPSandPPSBytes == NULL) return; // sanity check
+  unsigned i;
+  u_int8_t* ptr = SPSandPPSBytes;
+  u_int8_t* limit = &SPSandPPSBytes[numSPSandPPSBytes];
 
-    unsigned numSPSs = *ptr++; checkPtr;
-    unsigned i;
-    for (i = 0; i < numSPSs; ++i) {
-      unsigned spsSize = (*ptr++)<<8; checkPtr;
-      spsSize |= *ptr++; checkPtr;
-
-      if (spsSize > numBytesRemaining) return;
-      if (i == 0) { // save the first one
-	fSPSSize = spsSize;
-	fSPS = new u_int8_t[spsSize];
-	memmove(fSPS, ptr, spsSize);
-      }
-      ptr += spsSize;
+  unsigned numSPSs = (*ptr++)&0x1F; CHECK_PTR;
+  for (i = 0; i < numSPSs; ++i) {
+    unsigned spsSize = (*ptr++)<<8; CHECK_PTR;
+    spsSize |= *ptr++; CHECK_PTR;
+    
+    if (spsSize > NUM_BYTES_REMAINING) return;
+    u_int8_t nal_unit_type = ptr[0]&0x1F;
+    if (fSPS == NULL && nal_unit_type == 7/*sanity check*/) { // save the first one
+      fSPSSize = spsSize;
+      fSPS = new u_int8_t[spsSize];
+      memmove(fSPS, ptr, spsSize);
     }
-
-    unsigned numPPSs = *ptr++; checkPtr;
-    for (i = 0; i < numPPSs; ++i) {
-      unsigned ppsSize = (*ptr++)<<8; checkPtr;
-      ppsSize |= *ptr++; checkPtr;
-
-      if (ppsSize > numBytesRemaining) return;
-      if (i == 0) { // save the first one
-	fPPSSize = ppsSize;
-	fPPS = new u_int8_t[ppsSize];
-	memmove(fPPS, ptr, ppsSize);
-      }
-      ptr += ppsSize;
+    ptr += spsSize;
+  }
+  
+  unsigned numPPSs = (*ptr++)&0x1F; CHECK_PTR;
+  for (i = 0; i < numPPSs; ++i) {
+    unsigned ppsSize = (*ptr++)<<8; CHECK_PTR;
+    ppsSize |= *ptr++; CHECK_PTR;
+    
+    if (ppsSize > NUM_BYTES_REMAINING) return;
+    u_int8_t nal_unit_type = ptr[0]&0x1F;
+    if (fPPS == NULL && nal_unit_type == 8/*sanity check*/) { // save the first one
+      fPPSSize = ppsSize;
+      fPPS = new u_int8_t[ppsSize];
+      memmove(fPPS, ptr, ppsSize);
     }
-  } while (0);
+    ptr += ppsSize;
+  }
 }
 
 H264VideoMatroskaFileServerMediaSubsession
@@ -100,7 +98,7 @@ float H264VideoMatroskaFileServerMediaSubsession::duration() const { return fOur
 void H264VideoMatroskaFileServerMediaSubsession
 ::seekStreamSource(FramedSource* inputSource, double& seekNPT, double /*streamDuration*/, u_int64_t& /*numBytes*/) {
   // "inputSource" is a framer. *Its* source is the demuxed track that we seek on:
-  H264VideoStreamFramer* framer = (H264VideoStreamFramer*)inputSource;
+  H264VideoStreamDiscreteFramer* framer = (H264VideoStreamDiscreteFramer*)inputSource;
 
   MatroskaDemuxedTrack* demuxedTrack = (MatroskaDemuxedTrack*)(framer->inputSource());
   demuxedTrack->seekToTime(seekNPT);
@@ -117,8 +115,9 @@ FramedSource* H264VideoMatroskaFileServerMediaSubsession
   if (baseH264VideoSource == NULL) return NULL;
   
   // Create a framer for the Video stream:
-  H264VideoStreamFramer* framer = H264VideoStreamDiscreteFramer::createNew(envir(), baseH264VideoSource);
-  framer->setSPSandPPS(fSPS, fSPSSize, fPPS, fPPSSize);
+  H264VideoStreamDiscreteFramer* framer
+    = H264VideoStreamDiscreteFramer::createNew(envir(), baseH264VideoSource);
+  framer->setVPSandSPSandPPS(NULL, 0, fSPS, fSPSSize, fPPS, fPPSSize);
 
   return framer;
 }
