@@ -14,7 +14,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2013 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2014 Live Networks, Inc.  All rights reserved.
 // Common routines used by both RTSP clients and servers
 // Implementation
 
@@ -69,19 +69,27 @@ Boolean parseRTSPRequestString(char const* reqStr,
 			       unsigned& contentLength) {
   // This parser is currently rather dumb; it should be made smarter #####
 
-  // Read everything up to the first space as the command name:
-  Boolean parseSucceeded = False;
+  // "Be liberal in what you accept": Skip over any whitespace at the start of the request:
   unsigned i;
-  for (i = 0; i < resultCmdNameMaxSize-1 && i < reqStrSize; ++i) {
+  for (i = 0; i < reqStrSize; ++i) {
+    char c = reqStr[i];
+    if (!(c == ' ' || c == '\t' || c == '\r' || c == '\n' || c == '\0')) break;
+  }
+  if (i == reqStrSize) return False; // The request consisted of nothing but whitespace!
+
+  // Then read everything up to the next space (or tab) as the command name:
+  Boolean parseSucceeded = False;
+  unsigned i1 = 0;
+  for (; i1 < resultCmdNameMaxSize-1 && i < reqStrSize; ++i,++i1) {
     char c = reqStr[i];
     if (c == ' ' || c == '\t') {
       parseSucceeded = True;
       break;
     }
 
-    resultCmdName[i] = c;
+    resultCmdName[i1] = c;
   }
-  resultCmdName[i] = '\0';
+  resultCmdName[i1] = '\0';
   if (!parseSucceeded) return False;
 
   // Skip over the prefix of any "rtsp://" or "rtsp:/" URL that follows:
@@ -124,7 +132,7 @@ Boolean parseRTSPRequestString(char const* reqStr,
       // The URL suffix comes from [k1+1,k]
       // Copy "resultURLSuffix":
       unsigned n = 0, k2 = k1+1;
-      if (i <= k) { // There's a slash after "host" or "host:port"
+      if (k2 <= k) {
         if (k - k1 + 1 > resultURLSuffixMaxSize) return False; // there's no room
         while (k2 <= k) resultURLSuffix[n++] = reqStr[k2++];
       }
@@ -132,10 +140,10 @@ Boolean parseRTSPRequestString(char const* reqStr,
 
       // The URL 'pre-suffix' comes from [i+1,k1-1]
       // Copy "resultURLPreSuffix":
-      n = 0; k2 = i + 1;
-      if (i <= k) { // There's a slash after "host" or "host:port"
+      n = 0; k2 = i+1;
+      if (k2+1 <= k1) {
         if (k1 - i > resultURLPreSuffixMaxSize) return False; // there's no room
-        while (k2 <= k1 - 1) resultURLPreSuffix[n++] = reqStr[k2++];
+        while (k2 <= k1-1) resultURLPreSuffix[n++] = reqStr[k2++];
       }
       resultURLPreSuffix[n] = '\0';
       decodeURL(resultURLPreSuffix);
@@ -268,6 +276,30 @@ Boolean parseRangeHeader(char const* buf, double& rangeStart, double& rangeEnd,
   return parseRangeParam(fields, rangeStart, rangeEnd, absStartTime, absEndTime);
 }
 
+Boolean parseScaleHeader(char const* buf, float& scale) {
+  // Initialize the result parameter to a default value:
+  scale = 1.0;
+
+  // First, find "Scale:"
+  while (1) {
+    if (*buf == '\0') return False; // not found
+    if (_strncasecmp(buf, "Scale:", 6) == 0) break;
+    ++buf;
+  }
+
+  // Then, run through each of the fields, looking for ones we handle:
+  char const* fields = buf + 6;
+  while (*fields == ' ') ++fields;
+  float sc;
+  if (sscanf(fields, "%f", &sc) == 1) {
+    scale = sc;
+  } else {
+    return False; // The header is malformed
+  }
+
+  return True;
+}
+
 // Used to implement "RTSPOptionIsSupported()":
 static Boolean isSeparator(char c) { return c == ' ' || c == ',' || c == ';' || c == ':'; }
 
