@@ -14,7 +14,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2013 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2014 Live Networks, Inc.  All rights reserved.
 // A generic SIP client
 // Implementation
 
@@ -38,6 +38,18 @@ SIPClient* SIPClient
 		       verbosityLevel, applicationName);
 }
 
+void SIPClient::setUserAgentString(char const* userAgentName) {
+  if (userAgentName == NULL) return;
+
+  // Change the existing user agent header string:
+  char const* const formatStr = "User-Agent: %s\r\n";
+  unsigned const headerSize = strlen(formatStr) + strlen(userAgentName);
+  delete[] fUserAgentHeaderStr;
+  fUserAgentHeaderStr = new char[headerSize];
+  sprintf(fUserAgentHeaderStr, formatStr, userAgentName);
+  fUserAgentHeaderStrLen = strlen(fUserAgentHeaderStr);
+}
+
 SIPClient::SIPClient(UsageEnvironment& env,
 		     unsigned char desiredAudioRTPPayloadFormat,
 		     char const* mimeSubtype,
@@ -45,8 +57,9 @@ SIPClient::SIPClient(UsageEnvironment& env,
   : Medium(env),
     fT1(500000 /* 500 ms */),
     fDesiredAudioRTPPayloadFormat(desiredAudioRTPPayloadFormat),
-    fVerbosityLevel(verbosityLevel),
-    fCSeq(0), fURL(NULL), fURLSize(0),
+    fVerbosityLevel(verbosityLevel), fCSeq(0),
+    fUserAgentHeaderStr(NULL), fUserAgentHeaderStrLen(0),
+    fURL(NULL), fURLSize(0),
     fToTagStr(NULL), fToTagStrSize(0),
     fUserName(NULL), fUserNameSize(0),
     fInviteSDPDescription(NULL), fInviteSDPDescriptionReturned(NULL),
@@ -91,11 +104,7 @@ SIPClient::SIPClient(UsageEnvironment& env,
     }
   }
 
-  // Set various headers to be used in each request:
-  char const* formatStr;
-  unsigned headerSize;
-
-  // Set the "User-Agent:" header:
+  // Set the "User-Agent:" header to use in each request:
   char const* const libName = "LIVE555 Streaming Media v";
   char const* const libVersionStr = LIVEMEDIA_LIBRARY_VERSION_STRING;
   char const* libPrefix; char const* libSuffix;
@@ -105,14 +114,13 @@ SIPClient::SIPClient(UsageEnvironment& env,
     libPrefix = " (";
     libSuffix = ")";
   }
-  formatStr = "User-Agent: %s%s%s%s%s\r\n";
-  headerSize
-    = strlen(formatStr) + fApplicationNameSize + strlen(libPrefix)
-    + strlen(libName) + strlen(libVersionStr) + strlen(libSuffix);
-  fUserAgentHeaderStr = new char[headerSize];
-  sprintf(fUserAgentHeaderStr, formatStr,
+  unsigned userAgentNameSize
+    = fApplicationNameSize + strlen(libPrefix) + strlen(libName) + strlen(libVersionStr) + strlen(libSuffix) + 1;
+  char* userAgentName = new char[userAgentNameSize];
+  sprintf(userAgentName, "%s%s%s%s%s",
 	  applicationName, libPrefix, libName, libVersionStr, libSuffix);
-  fUserAgentHeaderStrSize = strlen(fUserAgentHeaderStr);
+  setUserAgentString(userAgentName);
+  delete[] userAgentName;
 
   reset();
 }
@@ -263,7 +271,7 @@ char* SIPClient::invite1(Authenticator* authenticator) {
       + 20 + fOurAddressStrSize
       + 20
       + strlen(authenticatorStr)
-      + fUserAgentHeaderStrSize
+      + fUserAgentHeaderStrLen
       + 20
       + inviteSDPSize;
     delete[] fInviteCmd; fInviteCmd = new char[inviteCmdSize];
