@@ -13,7 +13,7 @@ You should have received a copy of the GNU Lesser General Public License
 along with this library; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 **********/
-// Copyright (c) 1996-2013, Live Networks, Inc.  All rights reserved
+// Copyright (c) 1996-2014, Live Networks, Inc.  All rights reserved
 // LIVE555 Proxy Server
 // main program
 
@@ -22,6 +22,8 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 
 char const* progName;
 UsageEnvironment* env;
+UserAuthenticationDatabase* authDB = NULL;
+UserAuthenticationDatabase* authDBForREGISTER = NULL;
 
 // Default values of command-line parameters:
 int verbosityLevel = 0;
@@ -29,12 +31,24 @@ Boolean streamRTPOverTCP = False;
 portNumBits tunnelOverHTTPPortNum = 0;
 char* username = NULL;
 char* password = NULL;
+Boolean proxyREGISTERRequests = False;
+char* usernameForREGISTER = NULL;
+char* passwordForREGISTER = NULL;
+
+static RTSPServer* createRTSPServer(Port port) {
+  if (proxyREGISTERRequests) {
+    return RTSPServerWithREGISTERProxying::createNew(*env, port, authDB, authDBForREGISTER, 65, streamRTPOverTCP, verbosityLevel);
+  } else {
+    return RTSPServer::createNew(*env, port, authDB);
+  }
+}
 
 void usage() {
   *env << "Usage: " << progName
        << " [-v|-V]"
        << " [-t|-T <http-port>]"
        << " [-u <username> <password>]"
+       << " [-R] [-U <username-for-REGISTER> <password-for-REGISTER>]"
        << " <rtsp-url-1> ... <rtsp-url-n>\n";
   exit(1);
 }
@@ -102,6 +116,22 @@ int main(int argc, char** argv) {
       break;
     }
 
+    case 'U': { // specify a username and password to use to authenticate incoming "REGISTER" commands
+      if (argc < 4) usage(); // there's no argv[3] (for the "password")
+      usernameForREGISTER = argv[2];
+      passwordForREGISTER = argv[3];
+
+      if (authDBForREGISTER == NULL) authDBForREGISTER = new UserAuthenticationDatabase;
+      authDBForREGISTER->addUserRecord(usernameForREGISTER, passwordForREGISTER);
+      argv += 2; argc -= 2;
+      break;
+    }
+
+    case 'R': { // Handle incoming "REGISTER" requests by proxying the specified stream:
+      proxyREGISTERRequests = True;
+      break;
+    }
+
     default: {
       usage();
       break;
@@ -110,13 +140,17 @@ int main(int argc, char** argv) {
 
     ++argv; --argc;
   }
-  if (argc < 2) usage(); // there must be at least one "rtsp://" URL at the end 
+  if (argc < 2 && !proxyREGISTERRequests) usage(); // there must be at least one "rtsp://" URL at the end 
   // Make sure that the remaining arguments appear to be "rtsp://" URLs:
   int i;
   for (i = 1; i < argc; ++i) {
     if (strncmp(argv[i], "rtsp://", 7) != 0) usage();
   }
   // Do some additional checking for invalid command-line argument combinations:
+  if (authDBForREGISTER != NULL && !proxyREGISTERRequests) {
+    *env << "The '-U <username> <password>' option can be used only with -R\n";
+    usage();
+  }
   if (streamRTPOverTCP) {
     if (tunnelOverHTTPPortNum > 0) {
       *env << "The -t and -T options cannot both be used!\n";
@@ -126,23 +160,21 @@ int main(int argc, char** argv) {
     }
   }
 
-  UserAuthenticationDatabase* authDB = NULL;
 #ifdef ACCESS_CONTROL
   // To implement client access control to the RTSP server, do the following:
   authDB = new UserAuthenticationDatabase;
   authDB->addUserRecord("username1", "password1"); // replace these with real strings
-  // Repeat the above with each <username>, <password> that you wish to allow
-  // access to the server.
+      // Repeat this line with each <username>, <password> that you wish to allow access to the server.
 #endif
 
   // Create the RTSP server.  Try first with the default port number (554),
   // and then with the alternative port number (8554):
   RTSPServer* rtspServer;
   portNumBits rtspServerPortNum = 554;
-  rtspServer = RTSPServer::createNew(*env, rtspServerPortNum, authDB);
+  rtspServer = createRTSPServer(rtspServerPortNum);
   if (rtspServer == NULL) {
     rtspServerPortNum = 8554;
-    rtspServer = RTSPServer::createNew(*env, rtspServerPortNum, authDB);
+    rtspServer = createRTSPServer(rtspServerPortNum);
   }
   if (rtspServer == NULL) {
     *env << "Failed to create RTSP server: " << env->getResultMsg() << "\n";
@@ -170,6 +202,10 @@ int main(int argc, char** argv) {
     delete[] proxyStreamURL;
   }
 
+  if (proxyREGISTERRequests) {
+    *env << "(We handle incoming \"REGISTER\" requests on port " << rtspServerPortNum << ")\n";
+  }
+
   // Also, attempt to create a HTTP server for RTSP-over-HTTP tunneling.
   // Try first with the default HTTP port (80), and then with the alternative HTTP
   // port numbers (8000 and 8080).
