@@ -13,7 +13,7 @@ You should have received a copy of the GNU Lesser General Public License
 along with this library; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 **********/
-// Copyright (c) 1996-2013, Live Networks, Inc.  All rights reserved
+// Copyright (c) 1996-2014, Live Networks, Inc.  All rights reserved
 // A RTSP client application that opens a RTSP URL argument,
 // and extracts and records the data from each incoming RTP stream.
 //
@@ -29,6 +29,10 @@ Medium* createClient(UsageEnvironment& env, char const* url, int verbosityLevel,
   return ourRTSPClient = RTSPClient::createNew(env, url, verbosityLevel, applicationName, tunnelOverHTTPPortNum);
 }
 
+void assignClient(Medium* client) {
+  ourRTSPClient = (RTSPClient*)client;
+}
+
 void getOptions(RTSPClient::responseHandler* afterFunc) { 
   ourRTSPClient->sendOptionsCommand(afterFunc, ourAuthenticator);
 }
@@ -37,8 +41,8 @@ void getSDPDescription(RTSPClient::responseHandler* afterFunc) {
   ourRTSPClient->sendDescribeCommand(afterFunc, ourAuthenticator);
 }
 
-void setupSubsession(MediaSubsession* subsession, Boolean streamUsingTCP, RTSPClient::responseHandler* afterFunc) {
-  Boolean forceMulticastOnUnspecified = False;
+void setupSubsession(MediaSubsession* subsession, Boolean streamUsingTCP, Boolean forceMulticastOnUnspecified, RTSPClient::responseHandler* afterFunc) {
+  
   ourRTSPClient->sendSetupCommand(*subsession, afterFunc, False, streamUsingTCP, forceMulticastOnUnspecified, ourAuthenticator);
 }
 
@@ -54,6 +58,10 @@ void tearDownSession(MediaSession* session, RTSPClient::responseHandler* afterFu
   ourRTSPClient->sendTeardownCommand(*session, afterFunc, ourAuthenticator);
 }
 
+void setUserAgentString(char const* userAgentString) {
+  ourRTSPClient->setUserAgentString(userAgentString);
+}
+
 Boolean allowProxyServers = False;
 Boolean controlConnectionUsesTCP = True;
 Boolean supportCodecSelection = False;
