@@ -13,7 +13,7 @@ You should have received a copy of the GNU Lesser General Public License
 along with this library; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 **********/
-// Copyright (c) 1996-2013, Live Networks, Inc.  All rights reserved
+// Copyright (c) 1996-2014, Live Networks, Inc.  All rights reserved
 // A SIP client test program that opens a SIP URL argument,
 // and extracts the data from each incoming RTP stream.
 
@@ -51,6 +51,10 @@ Medium* createClient(UsageEnvironment& env, char const* /*url*/, int verbosityLe
   return ourSIPClient = SIPClient::createNew(env, desiredAudioRTPPayloadFormat, mimeSubtype, verbosityLevel, applicationName);
 }
 
+// The followign function is implemented, but is not used for "playSIP":
+void assignClient(Medium* /*client*/) {
+}
+
 void getOptions(RTSPClient::responseHandler* afterFunc) { 
   ourSIPClient->envir().setResultMsg("NOT SUPPORTED IN CLIENT");
   afterFunc(NULL, -1, strDup(ourSIPClient->envir().getResultMsg()));
@@ -93,7 +97,7 @@ void getSDPDescription(RTSPClient::responseHandler* afterFunc) {
   afterFunc(NULL, resultCode, strDup(result));
 }
 
-void setupSubsession(MediaSubsession* subsession, Boolean /*streamUsingTCP*/, RTSPClient::responseHandler* afterFunc) {
+void setupSubsession(MediaSubsession* subsession, Boolean /*streamUsingTCP*/, Boolean /*forceMulticastOnUnspecified*/,RTSPClient::responseHandler* afterFunc) {
   subsession->setSessionId("mumble"); // anything that's non-NULL will work
 
   ////////// BEGIN hack code that should really be implemented in SIPClient //////////
@@ -170,6 +174,10 @@ void tearDownSession(MediaSession* /*session*/, RTSPClient::responseHandler* aft
   }
 }
 
+void setUserAgentString(char const* userAgentString) {
+  ourSIPClient->setUserAgentString(userAgentString);
+}
+
 Boolean allowProxyServers = True;
 Boolean controlConnectionUsesTCP = False;
 Boolean supportCodecSelection = True;
